<?php

namespace Lc\CaracoleBundle\Model\User;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use Lc\CaracoleBundle\Model\Address\AddressInterface;
use Lc\CaracoleBundle\Model\Newsletter\NewsletterInterface;
use Lc\CaracoleBundle\Model\Order\OrderShopInterface;
use Lc\CaracoleBundle\Model\Product\ProductFamilyInterface;
use Lc\CaracoleBundle\Model\Reduction\ReductionCreditInterface;
use Lc\SovBundle\Model\User\User as SovUserModel;
use Lc\CaracoleBundle\Model\Merchant\MerchantInterface;
use Lc\CaracoleBundle\Model\Ticket\TicketInterface;
use Lc\SovBundle\Model\User\UserInterface;

/**
 * @ORM\MappedSuperclass()
 *
 */
abstract class UserModel extends SovUserModel
{
    /**
     * @ORM\Column(type="string", length=20, nullable=true)
     */
    protected $phone;

    /**
     * @ORM\Column(type="string", length=64, nullable=true)
     */
    protected $behaviorDisplayPrice;

    /**
     * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\Merchant\MerchantInterface")
     */
    protected $merchant;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Address\AddressInterface", mappedBy="user", cascade={"persist"})
     */
    protected $addresses;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Order\OrderShopInterface", mappedBy="user")
     */
    protected $orders;

    /**
     * @ORM\Column(type="string", length=64, nullable=true)
     */
    protected $firstname;

    /**
     * @ORM\Column(type="string", length=64, nullable=true)
     */
    protected $lastname;

    /**
     * @ORM\Column(type="boolean", nullable=true)
     */
    protected $gender;

    /**
     * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\Newsletter\NewsletterInterface", inversedBy="users")
     */
    protected $newsletters;

    /**
     * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\User\GroupUserInterface", inversedBy="users")
     */
    protected $groupUsers;

    /**
     * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\Product\ProductFamilyInterface")
     */
    protected $favoriteProductFamilies;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\User\UserPointSaleInterface", mappedBy="user", orphanRemoval=true)
     */
    protected $userPointSales;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\User\UserMerchantInterface", mappedBy="user", orphanRemoval=true)
     */
    protected $userMerchants;

    /**
     * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Ticket\TicketInterface", mappedBy="user")
     */
    protected $tickets;

    /**
     * @ORM\Column(type="array", nullable=true)
     */
    protected $ticketTypesNotification = [];


    /**
     * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\Reduction\ReductionCreditInterface", mappedBy="users")
     */
    protected $reductionCredits;


    public function __construct()
    {
        $this->addresses = new ArrayCollection();
        $this->orders = new ArrayCollection();
        $this->groupUsers = new ArrayCollection();
        $this->newsletters = new ArrayCollection();
        $this->favoriteProductFamilies = new ArrayCollection();
        $this->userPointSales = new ArrayCollection();
        $this->userMerchants = new ArrayCollection();
        $this->reductionCredits = new ArrayCollection();
        $this->tickets = new ArrayCollection();
    }

    public function __toString()
    {
        return $this->getSummary();
    }

    public function getSummary()
    {
        return '#' . $this->getId() . ' ' . strtoupper($this->getLastname()) . ' ' . $this->getFirstname(
                ) . ' (' . $this->getEmail() . ')';
    }

    public function getName(): ?string
    {
        return (string)ucfirst(strtolower($this->getFirstname())) . ' ' . strtoupper($this->getLastname());
    }

    public function getPhone(): ?string
    {
        return $this->phone;
    }

    public function setPhone(?string $phone): self
    {
        $this->phone = $phone;

        return $this;
    }

    public function getBehaviorDisplayPrice(): ?string
    {
        return $this->behaviorDisplayPrice;
    }

    public function setBehaviorDisplayPrice(?string $behaviorDisplayPrice): self
    {
        $this->behaviorDisplayPrice = $behaviorDisplayPrice;

        return $this;
    }

    public function getMerchant(): ?MerchantInterface
    {
        return $this->merchant;
    }

    public function setMerchant(?MerchantInterface $merchant): self
    {
        $this->merchant = $merchant;

        return $this;
    }

    /**
     * @return Collection|AddressInterface[]
     */
    public function getAddresses($status = null): Collection
    {
        if ($status) {
            $addressToReturn = new ArrayCollection();
            foreach ($this->addresses as $address) {
                if ($address->getStatus() == $status) {
                    $addressToReturn[] = $address;
                }
            }
            return $addressToReturn;
        } else {
            return $this->addresses;
        }
    }

    public function addAddress(AddressInterface $address): self
    {
        if (!$this->addresses->contains($address)) {
            $this->addresses[] = $address;
            $address->setUser($this);
        }

        return $this;
    }

    public function removeAddress(AddressInterface $address): self
    {
        if ($this->addresses->contains($address)) {
            $this->addresses->removeElement($address);
            // set the owning side to null (unless already changed)
            if ($address->getUser() === $this) {
                $address->setUser(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|OrderShopInterface[]
     */
    public function getOrders(): Collection
    {
        return $this->orders;
    }

    public function addOrder(OrderShopInterface $order): self
    {
        if (!$this->orders->contains($order)) {
            $this->orders[] = $order;
            $order->setUser($this);
        }

        return $this;
    }

    public function removeOrder(OrderShopInterface $order): self
    {
        if ($this->orders->contains($order)) {
            $this->orders->removeElement($order);
            // set the owning side to null (unless already changed)
            if ($order->getUser() === $this) {
                $order->setUser(null);
            }
        }

        return $this;
    }

    public function getFirstname(): ?string
    {
        return $this->firstname;
    }

    public function setFirstname(?string $firstname): self
    {
        $this->firstname = $firstname;

        return $this;
    }

    public function getLastname(): ?string
    {
        return $this->lastname;
    }

    public function setLastname(?string $lastname): self
    {
        $this->lastname = $lastname;

        return $this;
    }

    public function getGender(): ?bool
    {
        return $this->gender;
    }

    public function setGender(?bool $gender): self
    {
        $this->gender = $gender;

        return $this;
    }

    /**
     * @return Collection|NewsletterInterface[]
     */
    public function getNewsletters(): Collection
    {
        return $this->newsletters;
    }

    public function addNewsletter(NewsletterInterface $newsletter): self
    {
        if (!$this->newsletters->contains($newsletter)) {
            $this->newsletters[] = $newsletter;
        }

        return $this;
    }

    public function removeNewsletter(NewsletterInterface $newsletter): self
    {
        if ($this->newsletters->contains($newsletter)) {
            $this->newsletters->removeElement($newsletter);
        }

        return $this;
    }

    /**
     * @return Collection|GroupUserInterface[]
     */
    public function getGroupUsers(): Collection
    {
        return $this->groupUsers;
    }

    public function addGroupUser(GroupUserInterface $groupUser): self
    {
        if (!$this->groupUsers->contains($groupUser)) {
            $this->groupUsers[] = $groupUser;
            $groupUser->addUser($this);
        }

        return $this;
    }

    public function removeGroupUser(GroupUserInterface $groupUser): self
    {
        if ($this->groupUsers->contains($groupUser)) {
            $this->groupUsers->removeElement($groupUser);
            $groupUser->removeUser($this);
        }

        return $this;
    }

    /**
     * @return Collection|ProductFamilyInterface[]
     */
    public function getFavoriteProductFamilies(): Collection
    {
        return $this->favoriteProductFamilies;
    }

    public function addFavoriteProductFamily(ProductFamilyInterface $favoriteProductFamily): self
    {
        if (!$this->favoriteProductFamilies->contains($favoriteProductFamily)) {
            $this->favoriteProductFamilies[] = $favoriteProductFamily;
        }

        return $this;
    }

    public function removeFavoriteProductFamily(ProductFamilyInterface $favoriteProductFamily): self
    {
        if ($this->favoriteProductFamilies->contains($favoriteProductFamily)) {
            $this->favoriteProductFamilies->removeElement($favoriteProductFamily);
        }

        return $this;
    }

    /**
     * @return Collection|UserPointSaleInterface[]
     */
    public function getUserPointSales(): Collection
    {
        return $this->userPointSales;
    }

    public function addUserPointSale(UserPointSaleInterface $userPointSale): self
    {
        if (!$this->userPointSales->contains($userPointSale)) {
            $this->userPointSales[] = $userPointSale;
            $userPointSale->setUser($this);
        }

        return $this;
    }

    public function removeUserPointSale(UserPointSaleInterface $userPointSale): self
    {
        if ($this->userPointSales->contains($userPointSale)) {
            $this->userPointSales->removeElement($userPointSale);
            // set the owning side to null (unless already changed)
            if ($userPointSale->getUser() === $this) {
                $userPointSale->setUser(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|TicketInterface[]
     */
    public function getTickets(): Collection
    {
        return $this->tickets;
    }

    public function addTicket(TicketInterface $ticket): self
    {
        if (!$this->tickets->contains($ticket)) {
            $this->tickets[] = $ticket;
            $ticket->setUser($this);
        }

        return $this;
    }

    public function removeTicket(TicketInterface $ticket): self
    {
        if ($this->tickets->contains($ticket)) {
            $this->tickets->removeElement($ticket);
            // set the owning side to null (unless already changed)
            if ($ticket->getUser() === $this) {
                $ticket->setUser(null);
            }
        }

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): self
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(\DateTimeInterface $updatedAt): self
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }


    public function getTicketTypesNotification(): ?array
    {
        return $this->ticketTypesNotification;
    }

    public function setTicketTypesNotification(?array $ticketTypesNotification): self
    {
        $this->ticketTypesNotification = $ticketTypesNotification;

        return $this;
    }

    /**
     * @return Collection|ReductionCreditInterface[]
     */
    public function getReductionCredits(): Collection
    {
        return $this->reductionCredits;
    }

    public function addReductionCredit(ReductionCreditInterface $reductionCredit): self
    {
        if (!$this->reductionCredits->contains($reductionCredit)) {
            $this->reductionCredits[] = $reductionCredit;
            $reductionCredit->addUser($this);
        }

        return $this;
    }

    public function removeReductionCredit(ReductionCreditInterface $reductionCredit): self
    {
        if ($this->reductionCredits->contains($reductionCredit)) {
            $this->reductionCredits->removeElement($reductionCredit);
            $reductionCredit->removeUser($this);
        }

        return $this;
    }

}
